@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* NativeVFS.cpp:
-**  Copyright (C) 2018-2019 Mednafen Team
+**  Copyright (C) 2018-2021 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -69,16 +69,16 @@ Stream* NativeVFS::open(const std::string& path, const uint32 mode, const int do
 bool NativeVFS::mkdir(const std::string& path, const bool throw_on_exist)
 {
  if(path.find('\0') != std::string::npos)
-  throw MDFN_Error(EINVAL, _("Error creating directory \"%s\": %s"), path.c_str(), _("Null character in path."));
+  throw MDFN_Error(EINVAL, _("Error creating directory \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Null character in path."));
 
  #ifdef WIN32
  bool invalid_utf8;
- std::u16string u16path = UTF8_to_UTF16(path, &invalid_utf8, true);
+ auto tpath = Win32Common::UTF8_to_T(path, &invalid_utf8, true);
 
  if(invalid_utf8)
-  throw MDFN_Error(EINVAL, _("Error creating directory \"%s\": %s"), path.c_str(), _("Invalid UTF-8"));
+  throw MDFN_Error(EINVAL, _("Error creating directory \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Invalid UTF-8"));
 
- if(::_wmkdir((const wchar_t*)u16path.c_str()))
+ if(::_tmkdir((const TCHAR*)tpath.c_str()))
  #elif defined HAVE_MKDIR
   #if MKDIR_TAKES_ONE_ARG
    if(::mkdir(path.c_str()))
@@ -92,7 +92,7 @@ bool NativeVFS::mkdir(const std::string& path, const bool throw_on_exist)
   ErrnoHolder ene(errno);
 
   if(ene.Errno() != EEXIST || throw_on_exist)
-   throw MDFN_Error(ene.Errno(), _("Error creating directory \"%s\": %s"), path.c_str(), ene.StrError());
+   throw MDFN_Error(ene.Errno(), _("Error creating directory \"%s\": %s"), MDFN_strhumesc(path).c_str(), ene.StrError());
 
   return false;
  }
@@ -107,16 +107,16 @@ bool NativeVFS::unlink(const std::string& path, const bool throw_on_noent, const
   _exit(-1);
 
  if(path.find('\0') != std::string::npos)
-  throw MDFN_Error(EINVAL, _("Error unlinking \"%s\": %s"), path.c_str(), _("Null character in path."));
+  throw MDFN_Error(EINVAL, _("Error unlinking \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Null character in path."));
 
  #ifdef WIN32
  bool invalid_utf8;
- std::u16string u16path = UTF8_to_UTF16(path, &invalid_utf8, true);
+ auto tpath = Win32Common::UTF8_to_T(path, &invalid_utf8, true);
 
  if(invalid_utf8)
-  throw MDFN_Error(EINVAL, _("Error unlinking \"%s\": %s"), path.c_str(), _("Invalid UTF-8"));
+  throw MDFN_Error(EINVAL, _("Error unlinking \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Invalid UTF-8"));
 
- if(::_wunlink((const wchar_t*)u16path.c_str()))
+ if(::_tunlink((const TCHAR*)tpath.c_str()))
  #else
  if(::unlink(path.c_str()))
  #endif
@@ -126,7 +126,7 @@ bool NativeVFS::unlink(const std::string& path, const bool throw_on_noent, const
   if(ene.Errno() == ENOENT && !throw_on_noent)
    return false;
   else
-   throw MDFN_Error(ene.Errno(), _("Error unlinking \"%s\": %s"), path.c_str(), ene.StrError());
+   throw MDFN_Error(ene.Errno(), _("Error unlinking \"%s\": %s"), MDFN_strhumesc(path).c_str(), ene.StrError());
  }
 
  return true;
@@ -138,29 +138,29 @@ void NativeVFS::rename(const std::string& oldpath, const std::string& newpath, c
   _exit(-1);
 
  if(oldpath.find('\0') != std::string::npos || newpath.find('\0') != std::string::npos)
-  throw MDFN_Error(EINVAL, _("Error renaming \"%s\" to \"%s\": %s"), oldpath.c_str(), newpath.c_str(), _("Null character in path."));
+  throw MDFN_Error(EINVAL, _("Error renaming \"%s\" to \"%s\": %s"), MDFN_strhumesc(oldpath).c_str(), MDFN_strhumesc(newpath).c_str(), _("Null character in path."));
 
  #ifdef WIN32
  bool invalid_utf8_old;
  bool invalid_utf8_new;
- std::u16string u16oldpath = UTF8_to_UTF16(oldpath, &invalid_utf8_old, true);
- std::u16string u16newpath = UTF8_to_UTF16(newpath, &invalid_utf8_new, true);
+ auto toldpath = Win32Common::UTF8_to_T(oldpath, &invalid_utf8_old, true);
+ auto tnewpath = Win32Common::UTF8_to_T(newpath, &invalid_utf8_new, true);
 
  if(invalid_utf8_old || invalid_utf8_new)
-  throw MDFN_Error(EINVAL, _("Error renaming \"%s\" to \"%s\": %s"), oldpath.c_str(), newpath.c_str(), _("Invalid UTF-8"));
+  throw MDFN_Error(EINVAL, _("Error renaming \"%s\" to \"%s\": %s"), MDFN_strhumesc(oldpath).c_str(), MDFN_strhumesc(newpath).c_str(), _("Invalid UTF-8"));
 
- if(::_wrename((const wchar_t*)u16oldpath.c_str(), (const wchar_t*)u16newpath.c_str()))
+ if(::_trename((const TCHAR*)toldpath.c_str(), (const TCHAR*)tnewpath.c_str()))
  {
   ErrnoHolder ene(errno);
 
   // TODO/FIXME: Ensure oldpath and newpath don't refer to the same file via symbolic link.
   if(oldpath != newpath && (ene.Errno() == EACCES || ene.Errno() == EEXIST))
   {
-   if(::_wunlink((const wchar_t*)u16newpath.c_str()) || _wrename((const wchar_t*)u16oldpath.c_str(), (const wchar_t*)u16newpath.c_str()))
-    throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), oldpath.c_str(), newpath.c_str(), ene.StrError());
+   if(::_tunlink((const TCHAR*)tnewpath.c_str()) || ::_trename((const TCHAR*)toldpath.c_str(), (const TCHAR*)tnewpath.c_str()))
+    throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), MDFN_strhumesc(oldpath).c_str(), MDFN_strhumesc(newpath).c_str(), ene.StrError());
   }
   else
-   throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), oldpath.c_str(), newpath.c_str(), ene.StrError());
+   throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), MDFN_strhumesc(oldpath).c_str(), MDFN_strhumesc(newpath).c_str(), ene.StrError());
  }
  #else
 
@@ -168,7 +168,7 @@ void NativeVFS::rename(const std::string& oldpath, const std::string& newpath, c
  {
   ErrnoHolder ene(errno);
 
-  throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), oldpath.c_str(), newpath.c_str(), ene.StrError());
+  throw MDFN_Error(ene.Errno(), _("Error renaming \"%s\" to \"%s\": %s"), MDFN_strhumesc(oldpath).c_str(), MDFN_strhumesc(newpath).c_str(), ene.StrError());
  }
 
  #endif
@@ -177,18 +177,18 @@ void NativeVFS::rename(const std::string& oldpath, const std::string& newpath, c
 bool NativeVFS::finfo(const std::string& path, FileInfo* fi, const bool throw_on_noent)
 {
  if(path.find('\0') != std::string::npos)
-  throw MDFN_Error(EINVAL, _("Error getting file information for \"%s\": %s"), path.c_str(), _("Null character in path."));
+  throw MDFN_Error(EINVAL, _("Error getting file information for \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Null character in path."));
  //
  //
  #ifdef WIN32
  bool invalid_utf8;
- std::u16string u16path = UTF8_to_UTF16(path, &invalid_utf8, true);
+ auto tpath = Win32Common::UTF8_to_T(path, &invalid_utf8, true);
  struct _stati64 buf;
 
  if(invalid_utf8)
-  throw MDFN_Error(EINVAL, _("Error getting file information for \"%s\": %s"), path.c_str(), _("Invalid UTF-8"));
+  throw MDFN_Error(EINVAL, _("Error getting file information for \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Invalid UTF-8"));
 
- if(::_wstati64((const wchar_t*)u16path.c_str(), &buf))
+ if(::_tstati64((const TCHAR*)tpath.c_str(), &buf))
  #else
  struct stat buf;
  if(::stat(path.c_str(), &buf))
@@ -199,7 +199,7 @@ bool NativeVFS::finfo(const std::string& path, FileInfo* fi, const bool throw_on
   if(ene.Errno() == ENOENT && !throw_on_noent)
    return false;
 
-  throw MDFN_Error(ene.Errno(), _("Error getting file information for \"%s\": %s"), path.c_str(), ene.StrError());
+  throw MDFN_Error(ene.Errno(), _("Error getting file information for \"%s\": %s"), MDFN_strhumesc(path).c_str(), ene.StrError());
  }
 
  if(fi)
@@ -221,7 +221,7 @@ bool NativeVFS::finfo(const std::string& path, FileInfo* fi, const bool throw_on
 void NativeVFS::readdirentries(const std::string& path, std::function<bool(const std::string&)> callb)
 {
  if(path.find('\0') != std::string::npos)
-  throw MDFN_Error(EINVAL, _("Error reading directory entries from \"%s\": %s"), path.c_str(), _("Null character in path."));
+  throw MDFN_Error(EINVAL, _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Null character in path."));
  //
  //
 #ifdef WIN32
@@ -230,35 +230,35 @@ void NativeVFS::readdirentries(const std::string& path, std::function<bool(const
  //
  HANDLE dp = nullptr;
  bool invalid_utf8;
- std::u16string u16path = UTF8_to_UTF16(path + preferred_path_separator + '*', &invalid_utf8, true);
- WIN32_FIND_DATAW ded;
+ auto tpath = Win32Common::UTF8_to_T(path + preferred_path_separator + '*', &invalid_utf8, true);
+ WIN32_FIND_DATA ded;
 
  if(invalid_utf8)
-  throw MDFN_Error(EINVAL, _("Error reading directory entries from \"%s\": %s"), path.c_str(), _("Invalid UTF-8"));
+  throw MDFN_Error(EINVAL, _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), _("Invalid UTF-8"));
 
  try
  {
-  if(!(dp = FindFirstFileW((const wchar_t*)u16path.c_str(), &ded)))
+  if(!(dp = FindFirstFile((const TCHAR*)tpath.c_str(), &ded)))
   {
    const uint32 ec = GetLastError();
 
-   throw MDFN_Error(0, _("Error reading directory entries from \"%s\": %s"), path.c_str(), Win32Common::ErrCodeToString(ec).c_str());
+   throw MDFN_Error(0, _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), Win32Common::ErrCodeToString(ec).c_str());
   }
 
   for(;;)
   {
    //printf("%s\n", UTF16_to_UTF8((const char16_t*)ded.cFileName, nullptr, true).c_str());
-   if(!callb(UTF16_to_UTF8((const char16_t*)ded.cFileName, nullptr, true)))
+   if(!callb(Win32Common::T_to_UTF8(ded.cFileName, nullptr, true)))
     break;
    //
-   if(!FindNextFileW(dp, &ded))
+   if(!FindNextFile(dp, &ded))
    {
     const uint32 ec = GetLastError();
 
     if(ec == ERROR_NO_MORE_FILES)
      break;
     else
-     throw MDFN_Error(0, _("Error reading directory entries from \"%s\": %s"), path.c_str(), Win32Common::ErrCodeToString(ec).c_str());
+     throw MDFN_Error(0, _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), Win32Common::ErrCodeToString(ec).c_str());
    }
   }
   //
@@ -286,7 +286,7 @@ void NativeVFS::readdirentries(const std::string& path, std::function<bool(const
   {
    ErrnoHolder ene(errno);
 
-   throw MDFN_Error(ene.Errno(), _("Error reading directory entries from \"%s\": %s"), path.c_str(), ene.StrError());
+   throw MDFN_Error(ene.Errno(), _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), ene.StrError());
   }
   //
   for(;;)
@@ -300,7 +300,7 @@ void NativeVFS::readdirentries(const std::string& path, std::function<bool(const
     {
      ErrnoHolder ene(errno);
 
-     throw MDFN_Error(ene.Errno(), _("Error reading directory entries from \"%s\": %s"), path.c_str(), ene.StrError());     
+     throw MDFN_Error(ene.Errno(), _("Error reading directory entries from \"%s\": %s"), MDFN_strhumesc(path).c_str(), ene.StrError());     
     }    
     break;
    }
@@ -356,7 +356,7 @@ void NativeVFS::check_firop_safe(const std::string& path)
  {
   if(path[x] & 0x80)
   {
-   MDFN_printf(_("WARNING: Referenced path \"%s\" contains at least one 8-bit non-ASCII character; this may cause portability issues.\n"), path.c_str());
+   MDFN_printf(_("WARNING: Referenced path \"%s\" contains at least one 8-bit non-ASCII character; this may cause portability issues.\n"), MDFN_strhumesc(path).c_str());
    break;
   }
  }
@@ -421,7 +421,7 @@ void NativeVFS::check_firop_safe(const std::string& path)
 #endif
 
  if(unsafe_reason.size() > 0)
-  throw MDFN_Error(0, _("Referenced path \"%s\" (escaped) is potentially unsafe.  %s Refer to the documentation about the \"filesys.untrusted_fip_check\" setting.\n"), MDFN_strescape(path).c_str(), unsafe_reason.c_str());
+  throw MDFN_Error(0, _("Referenced path \"%s\" is potentially unsafe.  %s Refer to the documentation about the \"filesys.untrusted_fip_check\" setting.\n"), MDFN_strhumesc(path).c_str(), unsafe_reason.c_str());
 }
 
 void NativeVFS::get_file_path_components(const std::string &file_path, std::string* dir_path_out, std::string* file_base_out, std::string *file_ext_out)
@@ -437,4 +437,9 @@ void NativeVFS::get_file_path_components(const std::string &file_path, std::stri
  VirtualFS::get_file_path_components(file_path, dir_path_out, file_base_out, file_ext_out);
 }
 
+std::string NativeVFS::get_human_path(const std::string& path)
+{
+ return MDFN_sprintf(_("\"%s\""), MDFN_strhumesc(path).c_str());
+}
+
 }
