@@ -25,6 +25,8 @@
 #include "Stream.h"
 #include <functional>
 
+#include <mednafen/string/string.h>
+
 namespace Mednafen
 {
 
@@ -98,6 +100,7 @@ class VirtualFS
 
  virtual void readdirentries(const std::string& path, std::function<bool(const std::string&)> callb) = 0;
 
+ virtual std::string get_human_path(const std::string& path) = 0;
  //
  //
  //
@@ -121,6 +124,16 @@ class VirtualFS
  // Create any directories needed to create a file at file_path
  void create_missing_dirs(const std::string& file_path);
 
+ /* std::string get_canonical_ext(const std::string& file_path); */
+
+ // 'ext' must have a leading dot(.)
+ INLINE bool test_ext(const std::string& path, const char* ext) const
+ {
+  size_t ext_len = strlen(ext);
+
+  return (path.size() >= ext_len) && !MDFN_memazicmp(path.c_str() + (path.size() - ext_len), ext, ext_len);
+ }
+
  protected:
  
  const char preferred_path_separator;
