@@ -33,7 +33,7 @@ void Input_GameClosed(void) MDFN_COLD;
 
 void Input_Update(bool VirtualDevicesOnly = false, bool UpdateRapidFire = true);
 
-void Input_MakeSettings(std::vector <MDFNSetting> &settings);
+void Input_MakeSettings(void);
 
 void Input_NetplayLPMChanged(void);
 
