@@ -4,8 +4,8 @@
 namespace MDFN_IEN_GB
 {
 
-uint32 SOUND_Read(int ts, uint32_t addr);
-void SOUND_Write(int ts, uint32 addr, uint8 val);
+MDFN_FASTCALL uint32 SOUND_Read(int ts, uint32_t addr);
+MDFN_FASTCALL void SOUND_Write(int ts, uint32 addr, uint8 val);
 
 int32 SOUND_Flush(int ts, int16 *SoundBuf, const int32 MaxSoundFrames);
 void SOUND_Init(void) MDFN_COLD;
