@@ -7,7 +7,7 @@ namespace Mednafen
 void MDFN_SetBaseDirectory(const std::string& dir);
 std::string MDFN_GetBaseDirectory(void);
 
-void GetFileBase(const char *f);
+void MDFN_SetFileBase(const std::string& dir_path, const std::string& file_base, const std::string& file_ext);
 
 std::string MDFN_MakeFName(int type, int id1, const char *cd1);
 
@@ -19,12 +19,13 @@ typedef enum
  MDFNMKF_SAVBACK,
  MDFNMKF_CHEAT,
  MDFNMKF_PALETTE,
- MDFNMKF_IPS,
+ MDFNMKF_PATCH,
  MDFNMKF_MOVIE,
  MDFNMKF_SNAP_DAT,
  MDFNMKF_CHEAT_TMP,
  MDFNMKF_FIRMWARE,
- MDFNMKF_PGCONFIG
+ MDFNMKF_PGCONFIG,
+ MDFNMKF_PMCONFIG
 } MakeFName_Type;
 
 std::string MDFN_MakeFName(MakeFName_Type type, int id1, const char *cd1);
