@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* crc.h:
-**  Copyright (C) 2018 Mednafen Team
+**  Copyright (C) 2018-2021 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -25,7 +25,9 @@
 namespace Mednafen
 {
 
-uint16 crc16_ccitt(const void* data, const size_t len);
+NO_CLONE NO_INLINE uint16 crc16_ccitt(const void* data, const size_t len);
+NO_CLONE NO_INLINE uint32 crc32_cdrom_edc(const void* data, const size_t len);
 
+void crc_test(void);
 }
 #endif
