@@ -6,7 +6,44 @@
 namespace Mednafen
 {
 
-MDFN_HIDE extern std::vector<const MDFNGI*> MDFNSystems;
+//
+// Basic overview of order of calls into Mednafen code:
+//
+//  (MDFNI_printf()/MDFN_printf/MDFN_indent and Mednafen utility functions other than those in CDUtility.h and video/text.h may be called before MDFNI_Init()
+//   if necessary, as long as they're called in the same thread that will call MDFNI_Init())
+//
+//  MDFNI_Init()
+//  while(whatever)
+//  {
+//   MDFNI_AddSetting() (driver-side code can use MDFNSystems global after MDFNI_Init() as necessary to generate per-system driver-side settings)
+//  }
+//  MDFNI_InitFinalize(basedir)
+//
+//  MDFNI_LoadSettings()
+//
+//  MDFNI_SetSetting()	(such as for handling command-line arguments; may also be called later, up to before the last MDFNI_SaveSettings())
+//  (...)
+//
+//  while(running)
+//  {
+//   MDFNI_LoadGame()
+//
+//   MDFNI_SetInput()
+//
+//   while(game_loaded)
+//   {
+//    (MDFNI_Reset(), MDFNI_Power(), MDFNI_SaveState(), etc. as necessary)
+//
+//    MDFNI_Emulate()
+//   }
+//   MDFNI_CloseGame()
+//  }
+//
+//  MDFNI_SaveSettings()
+//
+//  MDFNI_Kill()
+//
+//
 
 /* Indent stdout newlines +- "indent" amount */
 void MDFN_indent(int indent);
@@ -16,7 +53,8 @@ struct MDFN_AutoIndent
  INLINE MDFN_AutoIndent(int amount) : indented(amount) { MDFN_indent(indented); }
  INLINE ~MDFN_AutoIndent() { MDFN_indent(-indented); }
 
- //INLINE void indent(int indoot) { indented += indoot; MDFN_indent(indoot); }
+ INLINE void adjust(int indoot) { indented += indoot; MDFN_indent(indoot); }
+ INLINE void reset(void) { MDFN_indent(-indented); indented = 0; }
  private:
  int indented;
 };
@@ -56,6 +94,19 @@ void MDFNI_Reset(void);
 void MDFNI_Power(void);
 
 
+// Allocates and initializes memory and other resources.
+// Call once, as soon as possible, before any Mednafen functions other than MDFN*_printf() and MDFN_indent.
+// Returns 'false' on error.
+bool MDFNI_Init(void) MDFN_COLD;
+
+// Only access after a successful call to MDFNI_Init()
+MDFN_HIDE extern std::vector<const MDFNGI*> MDFNSystems;
+
+// Finalizes settings, and sets the base directory.
+// Call once, after MDFNI_Init() and any MDFNI_AddSetting() calls.
+// Returns 'false' on error.
+bool MDFNI_InitFinalize(const char *basedir) MDFN_COLD;
+
 // path = path of game/file to load.
 // Returns NULL on error.
 //
@@ -72,21 +123,22 @@ MDFNGI* MDFNI_LoadGame(const char* force_module, VirtualFS* vfs, const char* pat
 class CDInterface;
 MDFNGI *MDFNI_LoadExternalCD(const char* force_module, const char* path_hint, CDInterface* cdif) MDFN_COLD;
 
-// Call this function as early as possible, even before MDFNI_Initialize()
-bool MDFNI_InitializeModules(void) MDFN_COLD;
-
-// Call once, after MDFNI_Initialize()
+// Loads settings from specified path.
+// Call once, after MDFNI_InitFinalize()
 // returns -1 if settings file didn't exist, 0 on error, and 1 on success
 int MDFNI_LoadSettings(const char* path);
 
+// Saves settings to specified path.
 // Call at least once right before MDFNI_Kill()
 bool MDFNI_SaveSettings(const char* path);
 
-/* allocates memory.  0 on failure, 1 on success. */
-/* Also pass it the base directory to load the configuration file. */
-int MDFNI_Initialize(const char *basedir, const std::vector<MDFNSetting> &DriverSettings) MDFN_COLD;
+// Saves settings to specified stream in a less user-friendly, compact form that's still compatible with
+// MDFNI_LoadSettings(), provided that the stream position is at 0 upon calling.  Intended to be used
+// for creating settings backups/snapshots.
+bool MDFNI_SaveSettingsCompact(Stream* s);
 
-/* Emulates a frame. */
+// Emulates a frame.
+// Call multiple times after MDFNI_LoadGame() and before MDFNI_CloseGame()
 void MDFNI_Emulate(EmulateSpecStruct *espec);
 
 #if 0
@@ -100,10 +152,11 @@ void MDFNI_Emulate(EmulateSpecStruct *espec);
 void MDFNI_AutoScaleMRFrame(EmulateSpecStruct *espec);
 #endif
 
-/* Closes currently loaded game */
+// Closes currently loaded game, freeing its associated resources.
 void MDFNI_CloseGame(void) MDFN_COLD;
 
-/* Deallocates all allocated memory.  Call after MDFNI_Emulate() returns. */
+// Frees (most) resources allocated by MDFNI_Init() and MDFNI_InitFinalize().
+// Call once, right before exit, and do not call any Mednafen functions afterward.
 void MDFNI_Kill(void) MDFN_COLD;
 
 void MDFN_DispMessage(const char *format, ...) noexcept MDFN_FORMATSTR(gnu_printf, 1, 2);
@@ -140,5 +193,34 @@ void MDFNI_StopWAVRecord(void) MDFN_COLD;
 
 void MDFNI_DumpModulesDef(const char *fn) MDFN_COLD;
 
+//
+//
+//
+void MDFNI_AddSetting(const MDFNSetting& s);
+
+// NULL-terminated
+void MDFNI_MergeSettings(const MDFNSetting* s);
+
+//
+// Due to how the per-module(and in the future, per-game) settings overrides work, we should
+// take care not to call MDFNI_SetSetting*() unless the setting has actually changed due to a user action.
+// I.E. do NOT call SetSetting*() unconditionally en-masse at emulator exit/game close to synchronize certain things like input mappings.
+//
+bool MDFNI_SetSetting(const char *name, const char *value, bool NetplayOverride = false);
+bool MDFNI_SetSetting(const char *name, const std::string& value, bool NetplayOverride = false);
+bool MDFNI_SetSetting(const std::string& name, const std::string& value, bool NetplayOverride = false);
+
+bool MDFNI_SetSettingB(const char *name, bool value);
+bool MDFNI_SetSettingB(const std::string& name, bool value);
+
+bool MDFNI_SetSettingUI(const char *name, uint64 value);
+bool MDFNI_SetSettingUI(const std::string& name, uint64 value);
+
+void MDFNI_DumpSettingsDef(const char *path);
+
+const std::vector<MDFNCS>* MDFNI_GetSettings(void);
+std::string MDFNI_GetSettingDefault(const char* name);
+std::string MDFNI_GetSettingDefault(const std::string& name);
+
 }
 #endif
