@@ -31,30 +31,30 @@ void (*X6502_Run)(int32 cycles);
 uint32 timestamp;
 void (MDFN_FASTCALL *MapIRQHook)(int a);
 
-#define _PC              X.PC
-#define _A               X.A
-#define _X               X.X
-#define _Y               X.Y
-#define _S               X.S
-#define _P               X.P
-#define _PI              X.mooPI
-#define _DB              X.DB
-#define _count           X.count
-#define _tcount          X.tcount
-#define _IRQlow          X.IRQlow
-#define _jammed          X.jammed
+#define CPU_PC              X.PC
+#define CPU_A               X.A
+#define CPU_X               X.X
+#define CPU_Y               X.Y
+#define CPU_S               X.S
+#define CPU_P               X.P
+#define CPU_PI              X.mooPI
+#define CPU_DB              X.DB
+#define CPU_count           X.count
+#define CPU_tcount          X.tcount
+#define CPU_IRQlow          X.IRQlow
+#define CPU_jammed          X.jammed
 
 #define ADDCYC(x)       \
 {       \
  int __x=x;     \
- _tcount+=__x;  \
- _count-=__x*48;        \
+ CPU_tcount+=__x;  \
+ CPU_count-=__x*48;        \
  timestamp+=__x;        \
 }
 
 static INLINE uint8 RdMemNorm(unsigned int A)
 {
- return(_DB=ARead[A](A));
+ return(CPU_DB=ARead[A](A));
 }
 
 static INLINE void WrMemNorm(unsigned int A, uint8 V)
@@ -63,16 +63,16 @@ static INLINE void WrMemNorm(unsigned int A, uint8 V)
 }
 
 #ifdef WANT_DEBUGGER
-X6502 XSave;     /* This is getting ugly. */
-//#define RdMemHook(A)	( X.ReadHook?(_DB=X.ReadHook(&X,A)):(_DB=ARead[A](A)) )
+static X6502 XSave;     /* This is getting ugly. */
+//#define RdMemHook(A)	( X.ReadHook?(CPU_DB=X.ReadHook(&X,A)):(CPU_DB=ARead[A](A)) )
 //#define WrMemHook(A,V)	{ if(X.WriteHook) X.WriteHook(&X,A,V); else BWrite[A](A,V); }
 
 static INLINE uint8 RdMemHook(unsigned int A)
 {
  if(X.ReadHook)
-  return(_DB = X.ReadHook(&X,A) );
+  return(CPU_DB = X.ReadHook(&X,A) );
  else
-  return(_DB=ARead[A](A));
+  return(CPU_DB=ARead[A](A));
 }
  
 static INLINE void WrMemHook(unsigned int A, uint8 V)
@@ -99,18 +99,18 @@ void X6502_DMW(uint32 A, uint8 V)
 #define PUSH(V) \
 {       \
  uint8 VTMP=V;  \
- WrMem(0x100+_S,VTMP);  \
- _S--;  \
+ WrMem(0x100+CPU_S,VTMP);  \
+ CPU_S--;  \
 }       
 
-#define POP() RdMem(0x100+(++_S))
+#define POP() RdMem(0x100+(++CPU_S))
 
 static uint8 ZNTable[256];
 /* Some of these operations will only make sense if you know what the flag
    constants are. */
 
-#define X_ZN(zort)      _P&=~(Z_FLAG|N_FLAG);_P|=ZNTable[zort]
-#define X_ZNT(zort)	_P|=ZNTable[zort]
+#define X_ZN(zort)      CPU_P&=~(Z_FLAG|N_FLAG);CPU_P|=ZNTable[zort]
+#define X_ZNT(zort)	CPU_P|=ZNTable[zort]
 
 #define JR(cond);	\
 {		\
@@ -118,93 +118,93 @@ static uint8 ZNTable[256];
  {	\
   uint32 tmp;	\
   int32 disp;	\
-  disp=(int8)RdMem(_PC);	\
-  _PC++;	\
+  disp=(int8)RdMem(CPU_PC);	\
+  CPU_PC++;	\
   ADDCYC(1);	\
-  tmp=_PC;	\
-  _PC+=disp;	\
-  if((tmp^_PC)&0x100)   \
+  tmp=CPU_PC;	\
+  CPU_PC+=disp;	\
+  if((tmp^CPU_PC)&0x100)   \
   ADDCYC(1);    \
 		\
-  ADDBT(_PC);	\
+  ADDBT(CPU_PC);	\
  }	\
- else _PC++;	\
+ else CPU_PC++;	\
 }
 
 #define ARNOP	   (void)x
 
-#define LDA	   _A=x;X_ZN(_A)
-#define LDX	   _X=x;X_ZN(_X)
-#define LDY        _Y=x;X_ZN(_Y)
+#define LDA	   CPU_A=x;X_ZN(CPU_A)
+#define LDX	   CPU_X=x;X_ZN(CPU_X)
+#define LDY        CPU_Y=x;X_ZN(CPU_Y)
 
 /*  All of the freaky arithmetic operations. */
-#define AND        _A&=x;X_ZN(_A)
-#define BIT        _P&=~(Z_FLAG|V_FLAG|N_FLAG);_P|=ZNTable[x&_A]&Z_FLAG;_P|=x&(V_FLAG|N_FLAG)
-#define EOR        _A^=x;X_ZN(_A)
-#define ORA        _A|=x;X_ZN(_A)
+#define AND        CPU_A&=x;X_ZN(CPU_A)
+#define BIT        CPU_P&=~(Z_FLAG|V_FLAG|N_FLAG);CPU_P|=ZNTable[x&CPU_A]&Z_FLAG;CPU_P|=x&(V_FLAG|N_FLAG)
+#define EOR        CPU_A^=x;X_ZN(CPU_A)
+#define ORA        CPU_A|=x;X_ZN(CPU_A)
 
 #define ADC  {	\
-	      uint32 l=_A+x+(_P&1);	\
-	      _P&=~(Z_FLAG|C_FLAG|N_FLAG|V_FLAG);	\
-              _P|=((((_A^x)&0x80)^0x80) & ((_A^l)&0x80))>>1;	\
-              _P|=(l>>8)&C_FLAG;	\
-	      _A=l;	\
-	      X_ZNT(_A);	\
+	      uint32 l=CPU_A+x+(CPU_P&1);	\
+	      CPU_P&=~(Z_FLAG|C_FLAG|N_FLAG|V_FLAG);	\
+              CPU_P|=((((CPU_A^x)&0x80)^0x80) & ((CPU_A^l)&0x80))>>1;	\
+              CPU_P|=(l>>8)&C_FLAG;	\
+	      CPU_A=l;	\
+	      X_ZNT(CPU_A);	\
 	     }
 
 #define SBC  {	\
-	      uint32 l=_A-x-((_P&1)^1);	\
-	      _P&=~(Z_FLAG|C_FLAG|N_FLAG|V_FLAG);	\
-	      _P|=((_A^l)&(_A^x)&0x80)>>1;	\
-	      _P|=((l>>8)&C_FLAG)^C_FLAG;	\
-	      _A=l;	\
-	      X_ZNT(_A);	\
+	      uint32 l=CPU_A-x-((CPU_P&1)^1);	\
+	      CPU_P&=~(Z_FLAG|C_FLAG|N_FLAG|V_FLAG);	\
+	      CPU_P|=((CPU_A^l)&(CPU_A^x)&0x80)>>1;	\
+	      CPU_P|=((l>>8)&C_FLAG)^C_FLAG;	\
+	      CPU_A=l;	\
+	      X_ZNT(CPU_A);	\
 	     }
 
 #define CMPL(a1,a2) {	\
 		     uint32 t=a1-a2;	\
 		     X_ZN(t&0xFF);	\
-		     _P&=~C_FLAG;	\
-		     _P|=((t>>8)&C_FLAG)^C_FLAG;	\
+		     CPU_P&=~C_FLAG;	\
+		     CPU_P|=((t>>8)&C_FLAG)^C_FLAG;	\
 		    }
 
 /* Special undocumented operation.  Very similar to CMP. */
 #define AXS	    {	\
-                     uint32 t=(_A&_X)-x;    \
+                     uint32 t=(CPU_A&CPU_X)-x;    \
                      X_ZN(t&0xFF);      \
-                     _P&=~C_FLAG;       \
-                     _P|=((t>>8)&C_FLAG)^C_FLAG;        \
-		     _X=t;	\
+                     CPU_P&=~C_FLAG;       \
+                     CPU_P|=((t>>8)&C_FLAG)^C_FLAG;        \
+		     CPU_X=t;	\
                     }
 
-#define CMP		CMPL(_A,x)
-#define CPX		CMPL(_X,x)
-#define CPY	      	CMPL(_Y,x)
+#define CMP		CMPL(CPU_A,x)
+#define CPX		CMPL(CPU_X,x)
+#define CPY	      	CMPL(CPU_Y,x)
 
 /* The following operations modify the byte being worked on. */
 #define DEC       	x--;X_ZN(x)
 #define INC		x++;X_ZN(x)
 
-#define ASL        _P&=~C_FLAG;_P|=x>>7;x<<=1;X_ZN(x)
-#define LSR	_P&=~(C_FLAG|N_FLAG|Z_FLAG);_P|=x&1;x>>=1;X_ZNT(x)
+#define ASL        CPU_P&=~C_FLAG;CPU_P|=x>>7;x<<=1;X_ZN(x)
+#define LSR	CPU_P&=~(C_FLAG|N_FLAG|Z_FLAG);CPU_P|=x&1;x>>=1;X_ZNT(x)
 
 /* For undocumented instructions, maybe for other things later... */
-#define LSRA	_P&=~(C_FLAG|N_FLAG|Z_FLAG);_P|=_A&1;_A>>=1;X_ZNT(_A)
+#define LSRA	CPU_P&=~(C_FLAG|N_FLAG|Z_FLAG);CPU_P|=CPU_A&1;CPU_A>>=1;X_ZNT(CPU_A)
 
 #define ROL	{	\
 		 uint8 l=x>>7;	\
 		 x<<=1;	\
-		 x|=_P&C_FLAG;	\
-		 _P&=~(Z_FLAG|N_FLAG|C_FLAG);	\
-		 _P|=l;	\
+		 x|=CPU_P&C_FLAG;	\
+		 CPU_P&=~(Z_FLAG|N_FLAG|C_FLAG);	\
+		 CPU_P|=l;	\
 		 X_ZNT(x);	\
 		}
 #define ROR	{	\
 		 uint8 l=x&1;	\
 		 x>>=1;	\
-		 x|=(_P&C_FLAG)<<7;	\
-		 _P&=~(Z_FLAG|N_FLAG|C_FLAG);	\
-		 _P|=l;	\
+		 x|=(CPU_P&C_FLAG)<<7;	\
+		 CPU_P&=~(Z_FLAG|N_FLAG|C_FLAG);	\
+		 CPU_P|=l;	\
 		 X_ZNT(x);	\
 		}
 		 
@@ -215,10 +215,10 @@ static uint8 ZNTable[256];
 /* Absolute */
 #define GetAB(target) 	\
 {	\
- target=RdMem(_PC);	\
- _PC++;	\
- target|=RdMem(_PC)<<8;	\
- _PC++;	\
+ target=RdMem(CPU_PC);	\
+ CPU_PC++;	\
+ target|=RdMem(CPU_PC)<<8;	\
+ CPU_PC++;	\
 }
 
 /* Absolute Indexed(for reads) */
@@ -248,24 +248,24 @@ static uint8 ZNTable[256];
 /* Zero Page */
 #define GetZP(target)	\
 {	\
- target=RdMem(_PC); 	\
- _PC++;	\
+ target=RdMem(CPU_PC); 	\
+ CPU_PC++;	\
 }
 
 /* Zero Page Indexed */
 #define GetZPI(target,i)	\
 {	\
- target=i+RdMem(_PC);	\
- _PC++;	\
+ target=i+RdMem(CPU_PC);	\
+ CPU_PC++;	\
 }
 
 /* Indexed Indirect */
 #define GetIX(target)	\
 {	\
  uint8 tmp;	\
- tmp=RdMem(_PC);	\
- _PC++;	\
- tmp+=_X;	\
+ tmp=RdMem(CPU_PC);	\
+ CPU_PC++;	\
+ tmp+=CPU_X;	\
  target=RdMem(tmp);	\
  tmp++;		\
  target|=RdMem(tmp)<<8;	\
@@ -276,13 +276,13 @@ static uint8 ZNTable[256];
 {	\
  unsigned int rt;	\
  uint8 tmp;	\
- tmp=RdMem(_PC);	\
- _PC++;	\
+ tmp=RdMem(CPU_PC);	\
+ CPU_PC++;	\
  rt=RdMem(tmp);	\
  tmp++;	\
  rt|=RdMem(tmp)<<8;	\
  target=rt;	\
- target+=_Y;	\
+ target+=CPU_Y;	\
  if((target^rt)&0x100)	\
  {	\
   RdMem(target - 0x100);	\
@@ -295,13 +295,13 @@ static uint8 ZNTable[256];
 {	\
  unsigned int rt;	\
  uint8 tmp;	\
- tmp=RdMem(_PC);	\
- _PC++;	\
+ tmp=RdMem(CPU_PC);	\
+ CPU_PC++;	\
  rt=RdMem(tmp);	\
  tmp++;	\
  rt|=RdMem(tmp)<<8;	\
  target=rt;	\
- target+=_Y;	\
+ target+=CPU_Y;	\
  RdMem((target&0x00FF)|(rt&0xFF00));	\
 }
 
@@ -310,34 +310,34 @@ static uint8 ZNTable[256];
    redundant) on the variable "x".
 */
 
-#define RMW_A(op) {uint8 x=_A; op; _A=x; break; } /* Meh... */
+#define RMW_A(op) {uint8 x=CPU_A; op; CPU_A=x; break; } /* Meh... */
 #define RMW_AB(op) {unsigned int A; uint8 x; GetAB(A); x=RdMem(A); WrMem(A,x); op; WrMem(A,x); break; }
 #define RMW_ABI(reg,op) {unsigned int A; uint8 x; GetABIWR(A,reg); x=RdMem(A); WrMem(A,x); op; WrMem(A,x); break; }
-#define RMW_ABX(op)	RMW_ABI(_X,op)
-#define RMW_ABY(op)	RMW_ABI(_Y,op)
+#define RMW_ABX(op)	RMW_ABI(CPU_X,op)
+#define RMW_ABY(op)	RMW_ABI(CPU_Y,op)
 #define RMW_IX(op)  {unsigned int A; uint8 x; GetIX(A); x=RdMem(A); WrMem(A,x); op; WrMem(A,x); break; }
 #define RMW_IY(op)  {unsigned int A; uint8 x; GetIYWR(A); x=RdMem(A); WrMem(A,x); op; WrMem(A,x); break; }
 #define RMW_ZP(op)  {uint8 A; uint8 x; GetZP(A); x=RdMem(A); op; WrMem(A,x); break; }
-#define RMW_ZPX(op) {uint8 A; uint8 x; GetZPI(A,_X); x=RdMem(A); op; WrMem(A,x); break;}
+#define RMW_ZPX(op) {uint8 A; uint8 x; GetZPI(A,CPU_X); x=RdMem(A); op; WrMem(A,x); break;}
 
-#define LD_IM(op)	{uint8 x; x=RdMem(_PC); _PC++; op; break;}
+#define LD_IM(op)	{uint8 x; x=RdMem(CPU_PC); CPU_PC++; op; break;}
 #define LD_ZP(op)	{uint8 A; uint8 x; GetZP(A); x=RdMem(A); op; break;}
-#define LD_ZPX(op)  {uint8 A; uint8 x; GetZPI(A,_X); x=RdMem(A); op; break;}
-#define LD_ZPY(op)  {uint8 A; uint8 x; GetZPI(A,_Y); x=RdMem(A); op; break;}
+#define LD_ZPX(op)  {uint8 A; uint8 x; GetZPI(A,CPU_X); x=RdMem(A); op; break;}
+#define LD_ZPY(op)  {uint8 A; uint8 x; GetZPI(A,CPU_Y); x=RdMem(A); op; break;}
 #define LD_AB(op)	{unsigned int A; uint8 x; GetAB(A); x=RdMem(A); op; break; }
 #define LD_ABI(reg,op)  {unsigned int A; uint8 x; GetABIRD(A,reg); x=RdMem(A); op; break;}
-#define LD_ABX(op)	LD_ABI(_X,op)
-#define LD_ABY(op)	LD_ABI(_Y,op)
+#define LD_ABX(op)	LD_ABI(CPU_X,op)
+#define LD_ABY(op)	LD_ABI(CPU_Y,op)
 #define LD_IX(op)	{unsigned int A; uint8 x; GetIX(A); x=RdMem(A); op; break;}
 #define LD_IY(op)	{unsigned int A; uint8 x; GetIYRD(A); x=RdMem(A); op; break;}
 
 #define ST_ZP(r)	{uint8 A; GetZP(A); WrMem(A,r); break;}
-#define ST_ZPX(r)	{uint8 A; GetZPI(A,_X); WrMem(A,r); break;}
-#define ST_ZPY(r)	{uint8 A; GetZPI(A,_Y); WrMem(A,r); break;}
+#define ST_ZPX(r)	{uint8 A; GetZPI(A,CPU_X); WrMem(A,r); break;}
+#define ST_ZPY(r)	{uint8 A; GetZPI(A,CPU_Y); WrMem(A,r); break;}
 #define ST_AB(r)	{unsigned int A; GetAB(A); WrMem(A,r); break;}
 #define ST_ABI(reg,r)	{unsigned int A; GetABIWR(A,reg); WrMem(A,r); break; }
-#define ST_ABX(r)	ST_ABI(_X,r)
-#define ST_ABY(r)	ST_ABI(_Y,r)
+#define ST_ABX(r)	ST_ABI(CPU_X,r)
+#define ST_ABY(r)	ST_ABI(CPU_Y,r)
 #define ST_IX(r)	{unsigned int A; GetIX(A); WrMem(A,r); break; }
 #define ST_IY(r)	{unsigned int A; GetIYWR(A); WrMem(A,r); break; }
 
@@ -363,35 +363,35 @@ static const uint8 CycTable[256] =
 
 void X6502_IRQBegin(int w)
 {
- //printf("IRQ begin: %d; %d\n", w, _IRQlow);
- _IRQlow|=w;
+ //printf("IRQ begin: %d; %d\n", w, CPU_IRQlow);
+ CPU_IRQlow|=w;
 }
 
 void X6502_IRQEnd(int w)
 {
- //printf("IRQ end: %d; %d\n", w, _IRQlow);
- _IRQlow&=~w;
+ //printf("IRQ end: %d; %d\n", w, CPU_IRQlow);
+ CPU_IRQlow&=~w;
 }
 
 void TriggerNMI(void)
 {
- _IRQlow|=MDFN_IQNMI;
+ CPU_IRQlow|=MDFN_IQNMI;
 }
 
 void TriggerNMI2(void)
 { 
- _IRQlow|=MDFN_IQNMI2;
+ CPU_IRQlow|=MDFN_IQNMI2;
 }
 
 /* Called from debugger. */
 void MDFNI_NMI(void)
 {
- _IRQlow|=MDFN_IQNMI;
+ CPU_IRQlow|=MDFN_IQNMI;
 }
 
 void MDFNI_IRQ(void)
 {
- _IRQlow|=MDFN_IQTEMP;
+ CPU_IRQlow|=MDFN_IQTEMP;
 }
 
 void X6502_GetIVectors(uint16 *reset, uint16 *irq, uint16 *nmi)
@@ -410,7 +410,7 @@ static int debugmode;
 
 void X6502_Reset(void)
 {
- _IRQlow=MDFN_IQRESET;
+ CPU_IRQlow=MDFN_IQRESET;
 }
   
 void X6502_Init(void)
@@ -429,7 +429,7 @@ void X6502_Init(void)
 
 void X6502_Power(void)
 {
- _count=_tcount=_IRQlow=_PC=_A=_X=_Y=_S=_P=_PI=_DB=_jammed=0;
+ CPU_count=CPU_tcount=CPU_IRQlow=CPU_PC=CPU_A=CPU_X=CPU_Y=CPU_S=CPU_P=CPU_PI=CPU_DB=CPU_jammed=0;
  timestamp=0;
  X6502_Reset();
 }
@@ -439,72 +439,72 @@ static void X6502_RunDebug(int32 cycles)
 {
 	#define RdMem RdMemHook
 	#define WrMem WrMemHook
-        #define ADDBT(to) NESDBG_AddBranchTrace(old_PC, to, 0)
+        #define ADDBT(to) NESDBG_AddBranchTrace(oldCPU_PC, to, 0)
 
         if(PAL)
          cycles*=15;          // 15*4=60
         else
          cycles*=16;          // 16*4=64
 
-        _count+=cycles;
+        CPU_count+=cycles;
 
 	PenguinPower:
-        while(_count>0)
+        while(CPU_count>0)
         {
-	 const uint16 old_PC = _PC;
+	 const uint16 oldCPU_PC = CPU_PC;
          int32 temp;
          uint8 b1;
 
-         if(_IRQlow && !X.cpoint) // Don't run IRQ stuff if we weren't here in a save state
+         if(CPU_IRQlow && !X.cpoint) // Don't run IRQ stuff if we weren't here in a save state
          { 
-          if(_IRQlow&MDFN_IQRESET)
+          if(CPU_IRQlow&MDFN_IQRESET)
           {
-	   _S -= 3;
-           _PC=RdMem(0xFFFC);
-           _PC|=RdMem(0xFFFD)<<8;
-	   NESDBG_AddBranchTrace(old_PC, _PC, 0xFFFC); //	   ADDBT(_PC);
-           _jammed=0;
-           _PI=_P=I_FLAG;
-           _IRQlow&=~MDFN_IQRESET;
+	   CPU_S -= 3;
+           CPU_PC=RdMem(0xFFFC);
+           CPU_PC|=RdMem(0xFFFD)<<8;
+	   NESDBG_AddBranchTrace(oldCPU_PC, CPU_PC, 0xFFFC); //	   ADDBT(CPU_PC);
+           CPU_jammed=0;
+           CPU_PI=CPU_P=I_FLAG;
+           CPU_IRQlow&=~MDFN_IQRESET;
           }
-          else if(_IRQlow&MDFN_IQNMI2)
+          else if(CPU_IRQlow&MDFN_IQNMI2)
           {
-           _IRQlow&=~MDFN_IQNMI2; 
-           _IRQlow|=MDFN_IQNMI;
+           CPU_IRQlow&=~MDFN_IQNMI2; 
+           CPU_IRQlow|=MDFN_IQNMI;
           }
-          else if(_IRQlow&MDFN_IQNMI)
+          else if(CPU_IRQlow&MDFN_IQNMI)
           {
-           if(!_jammed)
+           if(!CPU_jammed)
            {
             ADDCYC(7);
-            PUSH(_PC>>8);
-            PUSH(_PC);
-            PUSH((_P&~B_FLAG)|(U_FLAG));
-            _P|=I_FLAG; 
-            _PC=RdMem(0xFFFA); 
-            _PC|=RdMem(0xFFFB)<<8;
-	    NESDBG_AddBranchTrace(old_PC, _PC, 0xFFFA); //    ADDBT(_PC);
-            _IRQlow&=~MDFN_IQNMI;
+            PUSH(CPU_PC>>8);
+            PUSH(CPU_PC);
+            PUSH((CPU_P&~B_FLAG)|(U_FLAG));
+            CPU_P|=I_FLAG; 
+            CPU_PC=RdMem(0xFFFA); 
+            CPU_PC|=RdMem(0xFFFB)<<8;
+	    NESDBG_AddBranchTrace(oldCPU_PC, CPU_PC, 0xFFFA); //    ADDBT(CPU_PC);
+            CPU_IRQlow&=~MDFN_IQNMI;
            }
           }
           else 
           { 
-           if(!(_PI&I_FLAG) && !_jammed)
+           if(!(CPU_PI&I_FLAG) && !CPU_jammed)
            {
             ADDCYC(7);
-            PUSH(_PC>>8);
-            PUSH(_PC);
-            PUSH((_P&~B_FLAG)|(U_FLAG));
-            _P|=I_FLAG;
-            _PC=RdMem(0xFFFE);
-            _PC|=RdMem(0xFFFF)<<8;
-	    NESDBG_AddBranchTrace(old_PC, _PC, 0xFFFE);	//ADDBT(_PC);
+            PUSH(CPU_PC>>8);
+            PUSH(CPU_PC);
+            PUSH((CPU_P&~B_FLAG)|(U_FLAG));
+            CPU_P|=I_FLAG;
+            CPU_PC=RdMem(0xFFFE);
+            CPU_PC|=RdMem(0xFFFF)<<8;
+	    NESDBG_AddBranchTrace(oldCPU_PC, CPU_PC, 0xFFFE);	//ADDBT(CPU_PC);
            }
           }
-          _IRQlow&=~(MDFN_IQTEMP);
-          if(_count<=0) 
+          CPU_IRQlow&=~(MDFN_IQTEMP);
+          if(CPU_count<=0) 
           {
-           _PI=_P;
+           CPU_PI=CPU_P;
            return;
           } /* Should increase accuracy without a */
              /* major speed hit. */
@@ -520,8 +520,8 @@ static void X6502_RunDebug(int32 cycles)
 
 	  fceuindbg=1;
 	  X.preexec=1;
-          b1=RdMem(_PC);
-          _PC++;
+          b1=RdMem(CPU_PC);
+          CPU_PC++;
           switch(b1)
           {   
            #include "ops.h"
@@ -535,27 +535,27 @@ static void X6502_RunDebug(int32 cycles)
 
 	 X.cpoint = 1;
 
-	 if(X.CPUHook) X.CPUHook(_PC);
+	 if(X.CPUHook) X.CPUHook(CPU_PC);
 	 if(!X.cpoint)
 	  goto PenguinPower;
 	 X.cpoint = 0;
 
-         _PI = _P;
-         b1=RdMem(_PC);
+         CPU_PI = CPU_P;
+         b1=RdMem(CPU_PC);
          ADDCYC(CycTable[b1]);
 
-         temp=_tcount;
-         _tcount=0;
+         temp=CPU_tcount;
+         CPU_tcount=0;
          if(MapIRQHook) MapIRQHook(temp);
 
 	 MDFN_SoundCPUHook(temp);
 
-         _PC++;
+         CPU_PC++;
          switch(b1)
          {
           #include "ops.h"
          } 
-         _PC &= 0xFFFF;
+         CPU_PC &= 0xFFFF;
         }
         #undef RdMem
         #undef WrMem
@@ -574,77 +574,77 @@ void X6502_Run(int32 cycles)
 
         uint32 pbackus;
 
-        pbackus=_PC;
+        pbackus=CPU_PC;
 
-        #undef _PC
-        #define _PC pbackus
+        #undef CPU_PC
+        #define CPU_PC pbackus
 
 	if(PAL)
 	 cycles*=15;          // 15*4=60
 	else
 	 cycles*=16;          // 16*4=64
 
-	_count+=cycles;
+	CPU_count+=cycles;
 	
         // This handles the (rare) case of a save state being saved in step mode in the debugger
-        if(X.cpoint && _count > 0)
+        if(X.cpoint && CPU_count > 0)
         {
          X.cpoint = 0;
          goto SephirothBishie;
         }
 
-	while(_count>0)
+	while(CPU_count>0)
 	{
 	 int32 temp;
 	 static uint8 b1;
 
-	 if(_IRQlow)
+	 if(CPU_IRQlow)
 	 {
-	  if(_IRQlow&MDFN_IQRESET)
+	  if(CPU_IRQlow&MDFN_IQRESET)
 	  {
-	   _S -= 3;
-	   _PC=RdMem(0xFFFC);
-	   _PC|=RdMem(0xFFFD)<<8;
-	   _jammed=0;
-	   _PI=_P=I_FLAG;
-	   _IRQlow&=~MDFN_IQRESET;
+	   CPU_S -= 3;
+	   CPU_PC=RdMem(0xFFFC);
+	   CPU_PC|=RdMem(0xFFFD)<<8;
+	   CPU_jammed=0;
+	   CPU_PI=CPU_P=I_FLAG;
+	   CPU_IRQlow&=~MDFN_IQRESET;
 	  }
-	  else if(_IRQlow&MDFN_IQNMI2)
+	  else if(CPU_IRQlow&MDFN_IQNMI2)
  	  {
-	   _IRQlow&=~MDFN_IQNMI2; 
-	   _IRQlow|=MDFN_IQNMI;
+	   CPU_IRQlow&=~MDFN_IQNMI2; 
+	   CPU_IRQlow|=MDFN_IQNMI;
 	  }
-	  else if(_IRQlow&MDFN_IQNMI)
+	  else if(CPU_IRQlow&MDFN_IQNMI)
 	  {
-	   if(!_jammed)
+	   if(!CPU_jammed)
 	   {
 	    ADDCYC(7);
-	    PUSH(_PC>>8);
-	    PUSH(_PC);
-	    PUSH((_P&~B_FLAG)|(U_FLAG));
-	    _P|=I_FLAG;		
-	    _PC=RdMem(0xFFFA); 
-	    _PC|=RdMem(0xFFFB)<<8;
-	    _IRQlow&=~MDFN_IQNMI;
+	    PUSH(CPU_PC>>8);
+	    PUSH(CPU_PC);
+	    PUSH((CPU_P&~B_FLAG)|(U_FLAG));
+	    CPU_P|=I_FLAG;		
+	    CPU_PC=RdMem(0xFFFA); 
+	    CPU_PC|=RdMem(0xFFFB)<<8;
+	    CPU_IRQlow&=~MDFN_IQNMI;
 	   }
 	  }
 	  else
 	  { 
-	   if(!(_PI&I_FLAG) && !_jammed)
+	   if(!(CPU_PI&I_FLAG) && !CPU_jammed)
 	   {
 	    ADDCYC(7);
-	    PUSH(_PC>>8);
-	    PUSH(_PC);
-	    PUSH((_P&~B_FLAG)|(U_FLAG));
-	    _P|=I_FLAG;
-	    _PC=RdMem(0xFFFE);
-	    _PC|=RdMem(0xFFFF)<<8;
+	    PUSH(CPU_PC>>8);
+	    PUSH(CPU_PC);
+	    PUSH((CPU_P&~B_FLAG)|(U_FLAG));
+	    CPU_P|=I_FLAG;
+	    CPU_PC=RdMem(0xFFFE);
+	    CPU_PC|=RdMem(0xFFFF)<<8;
 	   }
 	  }
-	  _IRQlow&=~(MDFN_IQTEMP);
-	  if(_count<=0) 
+	  CPU_IRQlow&=~(MDFN_IQTEMP);
+	  if(CPU_count<=0) 
 	  {
-	   _PI=_P;
+	   CPU_PI=CPU_P;
 	   X.PC = pbackus;
 	   return;
 	   } /* Should increase accuracy without a  major speed hit. */
@@ -652,33 +652,33 @@ void X6502_Run(int32 cycles)
 
  	 SephirothBishie:
 
-	 _PI = _P;
-	 b1=RdMem(_PC);
+	 CPU_PI = CPU_P;
+	 b1=RdMem(CPU_PC);
 
 	 ADDCYC(CycTable[b1]);
 
-	 temp=_tcount;
-	 _tcount=0;
+	 temp=CPU_tcount;
+	 CPU_tcount=0;
 	 if(MapIRQHook) MapIRQHook(temp);
 	 MDFN_SoundCPUHook(temp);
-	 _PC++;
+	 CPU_PC++;
 
          switch(b1)
          {
           #include "ops.h"
          } 
-         _PC &= 0xFFFF;
+         CPU_PC &= 0xFFFF;
 	}
 
-        #undef _PC
-        #define _PC X.PC
+        #undef CPU_PC
+        #define CPU_PC X.PC
 	#undef ADDBT
-        _PC=pbackus;
+        CPU_PC=pbackus;
 }
 
 void X6502_StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
- extern uint8 RAM[0x800]; // from nes.cpp
+ MDFN_HIDE extern uint8 RAM[0x800]; // from nes.cpp
 
  SFORMAT SFCPU[]={
   SFVARN(X.PC, "PC"),
