@@ -171,9 +171,14 @@ void VCE::Reset(const int32 timestamp)
  lc263 = 0;
  bw = 0;
 
- memset(color_table, 0, sizeof(color_table));
  memset(color_table_cache, 0, sizeof(color_table_cache));
 
+ for(unsigned i = 0; i < 0x200; i++)
+ {
+  color_table[i] = ((i ^ (i >> 3)) & 1) ? 0x000 : 0x1FF;
+  FixPCache(i);
+ }
+
  ctaddress = 0;
 
  hblank = 1;
