@@ -18,7 +18,7 @@
 #include "pce.h"
 #include "vdc.h"
 
-namespace PCE_Fast
+namespace MDFN_IEN_PCE_FAST
 {
 
 static uint8 dummy_bank[8192 + 8192];  // + 8192 for PC-as-ptr safety padding
