@@ -12,14 +12,14 @@ using namespace Mednafen;
 #define DECLFR(x) uint8 MDFN_FASTCALL x (uint32 A)
 #define DECLFW(x) void MDFN_FASTCALL x (uint32 A, uint8 V)
 
-namespace PCE_Fast
+namespace MDFN_IEN_PCE_FAST
 {
-extern uint8 ROMSpace[0x88 * 8192 + 8192];
+MDFN_HIDE extern uint8 ROMSpace[0x88 * 8192 + 8192];
 
 typedef void (MDFN_FASTCALL *writefunc)(uint32 A, uint8 V);
 typedef uint8 (MDFN_FASTCALL *readfunc)(uint32 A);
 
-extern uint8 PCEIODataBuffer;
+MDFN_HIDE extern uint8 PCEIODataBuffer;
 
 void PCE_InitCD(void) MDFN_COLD;
 
@@ -27,18 +27,18 @@ void PCE_InitCD(void) MDFN_COLD;
 
 #include "huc6280.h"
 
-namespace PCE_Fast
+namespace MDFN_IEN_PCE_FAST
 {
-extern bool PCE_ACEnabled; // Arcade Card emulation enabled?
+MDFN_HIDE extern bool PCE_ACEnabled; // Arcade Card emulation enabled?
 void PCE_Power(void) MDFN_COLD;
 
-extern int pce_overclocked;
+MDFN_HIDE extern int pce_overclocked;
 
-extern uint8 BaseRAM[32768 + 8192];
+MDFN_HIDE extern uint8 BaseRAM[32768 + 8192];
 
 };
 
-using namespace PCE_Fast;
+using namespace MDFN_IEN_PCE_FAST;
 
 #define _PCE_H
 #endif
