@@ -444,9 +444,18 @@ static void CPUHandler(const v810_timestamp_t timestamp, uint32 PC)
    };
 
    // FIXME, overflow possible and speed
-   PCFXDBG_DoLog("ROMFONT", "0x%08x->0xFFF0000C, PR7=0x%08x=%s, PR6=0x%04x = %s", lastPC, PCFX_V810.GetPR(7), (PCFX_V810.GetPR(7) > 5) ? "?" : font_sizes[PCFX_V810.GetPR(7)], PCFX_V810.GetPR(6) & 0xFFFF, PCFXDBG_ShiftJIS_to_UTF8(PCFX_V810.GetPR(6) & 0xFFFF));
-   setvbuf(stdout, NULL, _IONBF, 0);
-   printf("%s", PCFXDBG_ShiftJIS_to_UTF8(PCFX_V810.GetPR(6) & 0xFFFF));
+   const uint32 pr7 = PCFX_V810.GetPR(7);
+   const uint16 sc = PCFX_V810.GetPR(6) & 0xFFFF;
+   const char* s = PCFXDBG_ShiftJIS_to_UTF8(sc);
+   PCFXDBG_DoLog("ROMFONT", "0x%08x->0xFFF0000C, PR7=0x%08x=%s, PR6=0x%04x = %s", lastPC, pr7, (pr7 > 5) ? "?" : font_sizes[pr7], sc, s);
+   for(const char* tmp = s; *tmp; tmp++)
+   {
+    const char c = *tmp;
+
+    if(c != 0x1B)
+     putchar(c);
+   }
+   fflush(stdout);
   }
   else if(PC == 0xFFF00008)
    DoSyscallLog();
@@ -1008,6 +1017,7 @@ static void ForceIRQ(int level)
 
 DebuggerInfoStruct PCFXDBGInfo =
 {
+ false,
  "shift_jis",
  4,
  2,             // Instruction alignment(bytes)
