@@ -41,7 +41,7 @@ namespace MDFN_IEN_PCFX
  #define FXDBG(format, ...) ((void)0)
 #endif
 
-extern V810 PCFX_V810;
+MDFN_HIDE extern V810 PCFX_V810;
 
 uint8 MDFN_FASTCALL mem_peekbyte(const v810_timestamp_t timestamp, const uint32 A);
 uint16 MDFN_FASTCALL mem_peekhword(const v810_timestamp_t timestamp, const uint32 A);
@@ -50,7 +50,7 @@ int32 MDFN_FASTCALL pcfx_event_handler(const v810_timestamp_t timestamp);
 
 void ForceEventUpdates(const uint32 timestamp);
 
-extern VDC *fx_vdc_chips[2];
+MDFN_HIDE extern VDC *fx_vdc_chips[2];
 
 #define REGSETHW(_reg, _data, _msh) { _reg &= 0xFFFF << (_msh ? 0 : 16); _reg |= _data << (_msh ? 16 : 0); }
 #define REGGETHW(_reg, _msh) ((_reg >> (_msh ? 16 : 0)) & 0xFFFF)
