@@ -597,12 +597,12 @@ uint32 NO_INLINE PS_CPU::Exception(uint32 code, uint32 PC, const uint32 NP, cons
 template<bool DebugMode, bool BIOSPrintMode, bool ILHMode>
 pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 {
- register pscpu_timestamp_t timestamp = timestamp_in;
+ pscpu_timestamp_t timestamp = timestamp_in;
 
- register uint32 PC;
- register uint32 new_PC;
- register uint32 LDWhich;
- register uint32 LDValue;
+ uint32 PC;
+ uint32 new_PC;
+ uint32 LDWhich;
+ uint32 LDValue;
  
  //printf("%d %d\n", gte_ts_done, muldiv_ts_done);
 
