@@ -161,7 +161,6 @@ struct PS_GPU
  bool InVBlank;
  bool sl_zero_reached;
  bool skip;
- bool hide_hoverscan;
  bool field;
  bool field_ram_readout;
  uint32 DisplayFB_CurYOffset;
@@ -197,6 +196,11 @@ struct PS_GPU
  MDFN_Rect *DisplayRect;
  int32 *LineWidths;
  int LineVisFirst, LineVisLast;
+ bool ShowHOverscan;
+ bool CorrectAspect;
+ int32 HVis;
+ int32 HVisOffs;
+ int32 NCABaseW;
  int32 hmc_to_visible;
  /*const*/ bool HardwarePALType;
  uint32 OutputLUT[384];
@@ -206,12 +210,14 @@ struct PS_GPU
  uint16 GPURAM[512][1024];
 };
 
- extern PS_GPU GPU;
+ MDFN_HIDE extern PS_GPU GPU;
 
  void GPU_Init(bool pal_clock_and_tv) MDFN_COLD;
  void GPU_Kill(void) MDFN_COLD;
 
- void GPU_SetGetVideoParams(MDFNGI* gi, const int sls, const int sle, const bool show_h_overscan) MDFN_COLD;
+ void GPU_SetGetVideoParams(MDFNGI* gi, const bool caspect, const int sls, const int sle, const bool show_h_overscan) MDFN_COLD;
+
+ void GPU_GetGunXTranslation(float* scale, float* offs);
 
  void GPU_Power(void) MDFN_COLD;
 
