@@ -137,25 +137,21 @@ InputDevice_Memcard::~InputDevice_Memcard()
 
 void InputDevice_Memcard::Power(void)
 {
+ presence_new = true;
+ memset(rw_buffer, 0, sizeof(rw_buffer));
+ write_xor = 0;
+
  dtr = 0;
-
- //buttons[0] = buttons[1] = 0;
-
  command_phase = 0;
-
  bitpos = 0;
-
  receive_buffer = 0;
 
  command = 0;
+ addr = 0;
+ calced_xor = 0;
 
  transmit_buffer = 0;
-
  transmit_count = 0;
-
- addr = 0;
-
- presence_new = true;
 }
 
 void InputDevice_Memcard::StateAction(StateMem* sm, const unsigned load, const bool data_only, const char* sname_prefix)
@@ -165,7 +161,7 @@ void InputDevice_Memcard::StateAction(StateMem* sm, const unsigned load, const b
  {
   SFVAR(presence_new),
 
-  SFARRAY(rw_buffer, sizeof(rw_buffer)),
+  SFVAR(rw_buffer),
   SFVAR(write_xor),
 
   SFVAR(dtr),
@@ -187,7 +183,7 @@ void InputDevice_Memcard::StateAction(StateMem* sm, const unsigned load, const b
 
  SFORMAT CD_StateRegs[] =
  {
-  SFARRAY(card_data, sizeof(card_data)),
+  SFVAR(card_data),
   SFEND
  };
 
