@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* sound.cpp - Sound Emulation
-**  Copyright (C) 2015-2017 Mednafen Team
+**  Copyright (C) 2015-2021 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #include <mednafen/mednafen.h>
 #include <mednafen/resampler/resampler.h>
 #include <mednafen/hw_cpu/m68k/m68k.h>
+#include <mednafen/jump.h>
 
 #ifndef MDFN_SSFPLAY_COMPILE
 #include "ss.h"
@@ -52,18 +53,20 @@ static int32 next_scsp_time;
 static uint32 clock_ratio;
 static sscpu_timestamp_t lastts;
 
+static MDFN_jmp_buf jbuf;
+
 static int16 IBuffer[1024][2];
 static uint32 IBufferCount;
 static SpeexResamplerState* resampler = NULL;
 static int last_rate;
 static uint32 last_quality;
 
-static INLINE void SCSP_SoundIntChanged(unsigned level)
+static INLINE void SCSP_SoundIntChanged(SS_SCSP* s, unsigned level)
 {
  SoundCPU.SetIPL(level);
 }
 
-static INLINE void SCSP_MainIntChanged(bool state)
+static INLINE void SCSP_MainIntChanged(SS_SCSP* s, bool state)
 {
  #ifndef MDFN_SSFPLAY_COMPILE
  SCU_SetInt(SCU_INT_SCSP, state);
@@ -203,6 +206,17 @@ static NO_INLINE void RunSCSP(void)
  bp[0] = (bp[0] * 27 + 16) >> 5;
  bp[1] = (bp[1] * 27 + 16) >> 5;
 
+/*
+ // TODO?  Need to measure frequency response more reliably first, ideally after capacitor
+ // replacement.  Should probably be controlled by a boolean setting, too.
+ for(unsigned lr = 0; lr < 2; lr++)
+ {
+  static int32 filt[2];
+  filt[lr] += (((int64)(int32)((uint32)bp[lr] << 16) - filt[lr]) * 60500) >> 16;
+  bp[lr] = filt[lr] >> 16;
+ }
+*/
+
  IBufferCount = (IBufferCount + 1) & 1023;
  next_scsp_time += 256;
 }
@@ -219,6 +233,8 @@ sscpu_timestamp_t SOUND_Update(sscpu_timestamp_t timestamp)
  lastts = timestamp;
  //
  //
+ MDFN_setjmp(jbuf);
+
  if(MDFN_LIKELY(SoundCPU.timestamp < (run_until_time >> 32)))
  {
   do
@@ -343,11 +359,22 @@ void SOUND_StateAction(StateMem* sm, const unsigned load, const bool data_only)
 
 //
 //
-// TODO: test masks.
 //
 template<typename T>
 static MDFN_FASTCALL T SoundCPU_BusRead(uint32 A)
 {
+ if(MDFN_UNLIKELY(A & (0xE00000 | (sizeof(T) - 1))))
+ {
+  SoundCPU.timestamp += 4;
+
+  if(A & (sizeof(T) - 1))
+   SoundCPU.SignalAddressError(A, 0x3);
+  else
+   SoundCPU.SignalDTACKHalted(A);
+
+  MDFN_longjmp(jbuf);
+ }
+ //
  T ret;
 
  SoundCPU.timestamp += 4;
@@ -364,6 +391,18 @@ static MDFN_FASTCALL T SoundCPU_BusRead(uint32 A)
 
 static MDFN_FASTCALL uint16 SoundCPU_BusReadInstr(uint32 A)
 {
+ if(MDFN_UNLIKELY(A & 0xE00001))
+ {
+  SoundCPU.timestamp += 4;
+
+  if(A & 1)
+   SoundCPU.SignalAddressError(A, 0x2);
+  else
+   SoundCPU.SignalDTACKHalted(A);
+
+  MDFN_longjmp(jbuf);
+ }
+ //
  uint16 ret;
 
  SoundCPU.timestamp += 4;
@@ -381,6 +420,18 @@ static MDFN_FASTCALL uint16 SoundCPU_BusReadInstr(uint32 A)
 template<typename T>
 static MDFN_FASTCALL void SoundCPU_BusWrite(uint32 A, T V)
 {
+ if(MDFN_UNLIKELY(A & (0xE00000 | (sizeof(T) - 1))))
+ {
+  SoundCPU.timestamp += 4;
+
+  if(A & (sizeof(T) - 1))
+   SoundCPU.SignalAddressError(A, 0x1);
+  else
+   SoundCPU.SignalDTACKHalted(A);
+
+  MDFN_longjmp(jbuf);
+ }
+ //
  SoundCPU.timestamp += 2;
 
  if(MDFN_UNLIKELY(SoundCPU.timestamp >= next_scsp_time))
@@ -395,6 +446,13 @@ static MDFN_FASTCALL void SoundCPU_BusWrite(uint32 A, T V)
 
 static MDFN_FASTCALL void SoundCPU_BusRMW(uint32 A, uint8 (MDFN_FASTCALL *cb)(M68K*, uint8))
 {
+ if(MDFN_UNLIKELY(A & 0xE00000))
+ {
+  SoundCPU.timestamp += 4;
+  SoundCPU.SignalDTACKHalted(A);
+  MDFN_longjmp(jbuf);
+ }
+ //
  uint8 tmp;
 
  SoundCPU.timestamp += 4;
@@ -437,6 +495,16 @@ void SOUND_SetSCSPRegister(const unsigned id, const uint32 value)
  SCSP.SetRegister(id, value);
 }
 
+uint32 SOUND_GetM68KRegister(const unsigned id, char* const special, const uint32 special_len)
+{
+ return SoundCPU.GetRegister(id, special, special_len);
+}
+
+void SOUND_SetM68KRegister(const unsigned id, const uint32 value)
+{
+ SoundCPU.SetRegister(id, value);
+}
+
 
 }
 
