@@ -71,12 +71,12 @@
 #include <map>
 #endif
 
-typedef int8_t int8;
+typedef signed char int8;
 typedef int16_t int16;
 typedef int32_t int32;
 typedef int64_t int64;
 
-typedef uint8_t uint8;
+typedef unsigned char uint8;	// Avoid uint8_t in case it's ever implemented as not being able to alias everything else, which Mednafen code relies on.
 typedef uint16_t uint16;
 typedef uint32_t uint32;
 typedef uint64_t uint64;
